@@ -57,17 +57,29 @@
        - adding new properties
        - displaying the list of properties in the set
        - changing the value of properties
+
+       Collection attributes:
+                  id: Name of the property inside the node object that is used
+                      to store the collection.
+              formId: Name of the controller variable that can be used to
+                      access the property collection form.
+              prompt: Label used to prompt the user to add properties
+                      to the collection.
+         placeholder: Label used to guide the user in providiing property
+                      values.
     */
     ctrl.propertyCollections = [
       {id: "properties",
-       title: "Properties",
-       addPrompt: "Add Property",
-       placeholder: "Property Name"
+       formId: "properties_form",
+       title: gettext("Properties"),
+       addPrompt: gettext("Add Property"),
+       placeholder: gettext("Property Name")
       },
       {id: "extra",
-       title: "Extras",
-       addPrompt: "Add Extra",
-       placeholder: "Extra Property Name"
+       formId: "extra_form",
+       title: gettext("Extras"),
+       addPrompt: gettext("Add Extra"),
+       placeholder: gettext("Extra Property Name")
       }];
 
     // Node object suitable for Ironic api
@@ -75,11 +87,14 @@
       name: null,
       driver: null,
       driver_info: {},
-      properties: {},
-      extra: {},
-      network_interface: null
+      network_interface: null,
+      resource_class: null
     };
 
+    angular.forEach(ctrl.propertyCollections, function(collection) {
+      ctrl.node[collection.id] = {};
+    });
+
     /**
      * @description Get the list of currently active Ironic drivers
      *
@@ -102,70 +117,6 @@
       });
     };
 
-    /**
-     * @description Check whether a group contains required properties
-     *
-     * @param {DriverProperty[]} group - Property group
-     * @return {boolean} Return true if the group contains required
-     * properties, false otherwise
-     */
-    function driverPropertyGroupHasRequired(group) {
-      var hasRequired = false;
-      for (var i = 0; i < group.length; i++) {
-        if (group[i].required) {
-          hasRequired = true;
-          break;
-        }
-      }
-      return hasRequired;
-    }
-
-    /**
-     * @description Convert array of driver property groups to a string
-     *
-     * @param {array[]} groups - Array for driver property groups
-     * @return {string} Output string
-     */
-    function driverPropertyGroupsToString(groups) {
-      var output = [];
-      angular.forEach(groups, function(group) {
-        var groupStr = [];
-        angular.forEach(group, function(property) {
-          groupStr.push(property.name);
-        });
-        groupStr = groupStr.join(", ");
-        output.push(['[', groupStr, ']'].join(""));
-      });
-      output = output.join(", ");
-      return ['[', output, ']'].join("");
-    }
-
-    /**
-     * @description Comaprison function used to sort driver property groups
-     *
-     * @param {DriverProperty[]} group1 - First group
-     * @param {DriverProperty[]} group2 - Second group
-     * @return {integer} Return:
-     * < 0 if group1 should precede group2 in an ascending ordering
-     * > 0 if group2 should precede group1
-     * 0 if group1 and group2 are considered equal from ordering perpsective
-     */
-    function compareDriverPropertyGroups(group1, group2) {
-      var group1HasRequired = driverPropertyGroupHasRequired(group1);
-      var group2HasRequired = driverPropertyGroupHasRequired(group2);
-
-      if (group1HasRequired === group2HasRequired) {
-        if (group1.length === group2.length) {
-          return group1[0].name.localeCompare(group2[0].name);
-        } else {
-          return group1.length - group2.length;
-        }
-      } else {
-        return group1HasRequired ? -1 : 1;
-      }
-      return 0;
-    }
-
     /**
      * @description Order driver properties in the form using the following
      * rules:
@@ -176,7 +127,7 @@
      * (2) Required properties with no dependents should be located at the
      * top of the form
      *
-     * @return {void}
+     * @return {[]} Ordered list of groups of strongly related properties
      */
     ctrl._sortDriverProperties = function() {
       // Build dependency graph between driver properties
@@ -221,10 +172,10 @@
           components.push(component);
         },
         groups);
-      groups.sort(compareDriverPropertyGroups);
+      groups.sort(baseNodeService.compareDriverPropertyGroups);
 
       $log.debug("Found the following property groups: " +
-                 driverPropertyGroupsToString(groups));
+                 baseNodeService.driverPropertyGroupsToString(groups));
       return groups;
     };
 
@@ -297,5 +248,28 @@
     ctrl.isDriverPropertyActive = function(property) {
       return property.isActive();
     };
+
+    /**
+     * @description Check whether the node definition form is ready for
+     *              to be submitted.
+     *
+     * @return {boolean} True if the form is ready to be submitted,
+     *                   otherwise false.
+     */
+    ctrl.readyToSubmit = function() {
+      var ready = true;
+      if (ctrl.driverProperties) {
+        for (var i = 0; i < ctrl.propertyCollections.length; i++) {
+          var collection = ctrl.propertyCollections[i];
+          if (ctrl[collection.formId].$invalid) {
+            ready = false;
+            break;
+          }
+        }
+      } else {
+        ready = false;
+      }
+      return ready;
+    };
   }
 })();
